
-- Base class for Spluttons

local AceOO = AceLibrary("AceOO-2.0")
--local ui = AceLibrary("uiQuery-1.0")

Splutton = AceOO.Class()

local self = Splutton

-- Pool of frames to reuse.
Splutton.framePool = setmetatable({}, 
						( {__index = function(self, n)
										if n==0 then
											return table.remove(self) or {}
										end
									end})
						)

local function getFrame() 
	return Splutton.framePool[0]
end
						
local function returnFrame(frame)
	for i,v in ipairs(frame) do
		frame[i] = nil
	end
	table.insert(Splutton.framePool, frame)
end
Splutton.Info = { "Type", "Texture", "SpellID", "MacroID" }
Splutton.Count = 0
Splutton.StateList = 0

Splutton.prototype.Parent = nil
Splutton.prototype.Children = nil
Splutton.prototype.X = 0
Splutton.prototype.Y = 0
Splutton.prototype.Type = "main" -- "Spell" "Macro" "Item"
Splutton.prototype.Value = nil
Splutton.prototype.Texture = "Interface\\Icons\\INV_Misc_QuestionMark"
Splutton.prototype.Count = 0

--[[
-- Splutton static functions to deal with each chain specifically
--]]

-- We need a helper function to create the chain of buttons so we can process them afterwards for state headers.
function Splutton:Setup(SplutType, Data)
	local Splutt= SplutType:new(nil, Data)
	-- Create a State Header (External State driver)
	Splutt.Header = CreateFrame("Frame", "SpluttonHeader", nil, "SecureStateHeaderTemplate")
	Splutt:SetHeaders(Splutt.Header)
	Splutt:SetNewStates()
	Splutt.Button:Show()
	return Splutt
end


-- Splutton class functions dealing with individual Spluttons
function Splutton.prototype:init(Parent, Data)
	Splutton.super.prototype.init(self)
	self.Parent = Parent

	for i,v in ipairs(Splutton.Info) do
		if Data[v] then
			self[v] = Data[v]
		end
	end
	

	if Data.Children then
		self.Children = {}
		for i,v in ipairs(Data.Children) do
			local splutKid = self.class:new(self, v)
			splutKid:Setup()
			table.insert(self.Children, splutKid)
		end
	end

	self:CreateButton()
end

function Splutton.prototype:CreateButton()
	if not self.Button then
		local splutton
		if self.Type == "main" then
			splutton = CreateFrame("CheckButton", "SpluttonHub"..Splutton.Count, self.Parent and self.Parent.Button or UIParent, "SecureAnchorButtonTemplate")
		else
			splutton = CreateFrame("Button", "Splutton"..Splutton.Count, self.Parent and self.Parent.Button or UIParent, "SecureActionButtonTemplate")
		end
		splutton:SetPoint("CENTER")
		self.Count = Splutton.Count
		Splutton.Count = Splutton.Count + 1
		splutton:RegisterForClicks("LeftButtonUp", "RightButtonUp")
		splutton:RegisterForDrag("LeftButton")
		splutton:SetMovable(true)
		splutton:SetWidth(48)
		splutton:SetHeight(48)
		splutton:SetBackdropColor(0.2, 0, 0, 0.3)
		splutton:SetScript("OnEnter", function() self.Setup(self,true) end)
		splutton:SetScript("OnDragStart", function(this)
			this:StartMoving()
		end)
		splutton:SetScript("OnDragStop", function(this)
		this:StopMovingOrSizing() end)
--		splutton:Hide()

		self.Button = splutton
		self.Button.self = self

	end
	if self.Type == "main" then -- Okay, now hook a StateHeader into the main button
		
		Splutton.StateList = Splutton.StateList + 1
		
		self.State = Splutton.StateList
	
--[[		-- Create a State Header (External State driver)
		self.Header = CreateFrame("Frame", "SpluttonHeader"..self.State, nil, "SecureStateHeaderTemplate")
		-- Attach it to our hub button	
		self.Header:SetAllPoints(self.Button) 
		self.Header:SetAttribute("addchild", self.Button)
		-- Make the hub button 'raise' any and all events on the children as well
		self.Button:SetAttribute("*childraise*", true)
		-- Pass the state onto any children and trigger the click event
		self.Button:SetAttribute("*childstate*", "^click") 
		-- On hub click, pass the following statemap to the header
		self.Header:SetAttribute("statemap-anchor-click", (Splutton.StateList+1).."-"..Splutton.StateList)
		-- and link the hub to the header
		self.Button:SetAttribute("anchorchild", "$parent") 
	--	self.Button:SetAttribute("childreparent", true)

		self.Button:SetAttribute("state", 0)
	--	self.Button:SetAttribute("statemap-parent", "$input")
	--	self.Button:SetAttribute("exportstate", "$input")
		
	--	self.Header:SetAttribute("statemap-parent", "$input")
	--	self.Header:SetAttribute("exportstate", "$input")
		if self.Children then
			for i,v in ipairs(self.Children) do
				self.Header:SetAttribute("addchild", v.Button)
				v.Button:SetAttribute("showstates", Splutton.StateList)
	--			v.Button:SetAttribute("statemap-parent","$input")
	--			if v.Type == "main" then
	--				self.Header:SetAttribute("addchild", v.Header)
	--			end
			end
		end
--]]		
	end
end

function Splutton.prototype:Setup(new)
	if not self.Button or self.Type == "main" or not (new or self.Value) then return end
	local itype,id, tag
	itype, id, tag = GetCursorInfo()
	
	if not itype then return end

	self.Type = itype
	if self.Type == "spell" then
		self.Value = GetSpellName(id, tag)
		self.Texture = GetSpellTexture(id, tag)
	end
	if self.Type == "item" then
		self.Value = GetItemInfo(id)
		self.Texture = select(10, GetItemInfo(id))
	end
	if self.Type == "macro" then
		self.Value = id
		_, self.Texture = GetMacroInfo(id)
	end
	self.Button:SetAttribute("unit","target")
	self.Button:SetAttribute("*type1", self.Type)
	self.Button:SetAttribute(self.Type.."1", self.Value)
	
	if self.Texture then
		_G[self.Button:GetName().."Icon"]:SetTexture(self.Texture)
	end

end

function Splutton.prototype:ToString()
	return self.Type..self.Count
end

-- This function traverses the button chains and applies the appropriate viewstates to 
-- each 'main' button so it will be 'seen' in each chain
function Splutton.prototype:SetHeaders(Header)
	Chain = self.State
	if self.Children then
		for i,v in ipairs(self.Children) do
			if v.Type == "main" then
				Chain = Chain ..","..v:SetHeaders(Header)
			else
				v.Button:SetAttribute("showstates", self.State)
				Header:SetAttribute("addchild", v.Button)
			end
		end
	end
	if self.Type == "main" then
		if not self.Parent then Chain = Chain ..",0" end
		self.Button:SetAttribute("showstates", Chain)
		Splut:Print("%s's viewstates are %s", self:ToString(), Chain)
	end
	self.Chain = Chain
	Header:SetAttribute("addchild", self.Button)
	return Chain
end

function Splutton.prototype:SetNewStates()
	for i,v in ipairs(self.Children) do
		if v.Type == "main" then
			local statemap = v.Parent.State..","..v.State
			Splut:Print("%s's statemap is %s",self:ToString(), statemap)
			v.Button:SetAttribute("newstate",statemap)
			if v.Children then
				v:SetNewStates()
			end
		end
	end
end
--]]
